#  $Copyright: Copyright (c) 2012 Symantec Corporation.
#  All rights reserved.
#
#  THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#  SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#  WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#  The Licensed Software and Documentation are deemed to be commercial
#  computer software as defined in FAR 12.212 and subject to restricted
#  rights as defined in FAR Section 52.227-19 "Commercial Computer
#  Software - Restricted Rights" and DFARS 227.7202, "Rights in
#  Commercial Computer Software or Commercial Computer Software
#  Documentation", as applicable, and any successor regulations. Any use,
#  modification, reproduction release, performance, display or disclosure
#  of the Licensed Software and Documentation by the U.S. Government
#  shall be solely in accordance with the terms of this Agreement.  $
#  $RCSfile: EVACAAgent.pm,v $ $Revision: 1.1.2.18 $
eval 'exec /opt/VRTSperl/bin/perl -I `pwd`/../../lib -S $0 ${1+"$@"}'
	if 0;

package EVACAAgent;
use ArrayRep;
@ISA = ("ArrayRep");
use ag_i18n_inc;
use File::Basename;
use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);

@EXPORT = qw(
);
$VERSION = '1.00';

# Replication Role
use constant ROLE_SOURCE => 0;
use constant ROLE_DESTINATION => 1;
use constant ROLE_UNKNOWN => -1;

# Replication Mode
use constant MODE_SYNC => 0;
use constant MODE_ASYNC => 1;
use constant MODE_UNKNOWN => -1;

# failover_required() return codes
use constant FR_NO => 0;
use constant FR_YES => 1;
use constant FR_UNKNOWN => -1;

# failover() return codes
use constant F_SUCCESS => 100;
use constant F_FAILURE => 101;

# General return codes
use constant SUCCESS => 0;
use constant ERROR => 1;

my $VXDCTL = "/usr/sbin/vxdctl";

#Scan devices
my $DEVFSADM = "/usr/sbin/devfsadm";
my $IOSCAN = "/usr/sbin/ioscan";
my $CFGMGR = "/usr/sbin/cfgmgr";

my $OSTYPE = "";
my $WINUSRPROFILE = "";

my $DEVNULL = "/dev/null";
if ($^O =~ /aix/i ){
	$OSTYPE = "AIX";
} elsif ($^O =~ /hpux/i ){
	$OSTYPE = "HPUX";
}elsif ($^O =~ /linux/i ){
	$OSTYPE = "LINUX";
}elsif ($^O =~ /solaris/i ){
	$OSTYPE = "SOLARIS";
}elsif ($^O =~ /MSWin32/i ){
	$OSTYPE = "WINDOWS";
	$DEVNULL = "nul";	

	#determine the user profile to be used according to the OS version.
	my @version = split('\s+' , `ver`);
	my $domain = Win32::DomainName();
	if($domain ne "") {
		$domain = "." . $domain;
	}
	#if version no. starts with 6, its win2k8
	if($version[4] =~ /^6/) {
		$WINUSRPROFILE = $ENV{"SYSTEMDRIVE"} . "\\Users\\Administrator" . $domain;		
	} else {
		$WINUSRPROFILE = $ENV{"SYSTEMDRIVE"} . "\\Documents and Settings\\Administrator" . $domain;		
	}
}


my $vcs_home=$VCSHOME;

%default = (
	resname => '',
	drgroupname	=> '',
	sssupath	=> '',
	role	=> '',
	mode	=> '',
	status  => '',
	splittakeover => '',
	autotakeover => '',
	mgmtserver => '',
	username => '',
	localevaname => '',
	remoteevaname => '',
	resource_type => 'EVACA',
);

sub new {
	my($proto, $resname, $href) = @_;
	my $class = ref($proto) || $proto;
	my $self = { %default };
	$self->{'resname'} = $resname;
	$self->{'drgroupname'} = $href->{'DRGroupName'};
	$self->{'sssupath'} = $href->{'SSSUPath'};
	$self->{'splittakeover'} = $href->{'SplitTakeover'};
	$self->{'autotakeover'} = $href->{'AutoTakeover'};
	$self->{'username'} = $href->{'UserName'};
	$self->{'mgmtserver'} = $href->{'ManagementServer'};
	$self->{'standbyusername'} = $href->{'StandbyUserName'};
	$self->{'standbymgmtserver'} = $href->{'StandbyManagementServer'};
	$self->{'localevaname'} = $href->{'LocalEVAName'};
	$self->{'role'} = ROLE_UNKNOWN;
	$self->{'mode'} = MODE_UNKNOWN;
	$self->{'remoteevaname'} = "";
	$self->{'start_time'} = $^T;
	bless($self, $class);
	return $self;
}

sub DESTROY {}

sub validate_args {
	my $href = shift;
	my $ret = SUCCESS; 
	
	my $sssupath;
	($ret, $sssupath) = VCSAG_GET_ATTR_VALUE ("SSSUPath", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute SSSUPath is not configured properly", 1);
		return ERROR;
	}
	$href->{"SSSUPath"} = $sssupath ;
	
	my $drgroupname;
	($ret, $drgroupname) = VCSAG_GET_ATTR_VALUE ("DRGroupName", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute DRGroupName is not configured properly", 2);
		return ERROR;
	}
	$href->{"DRGroupName"} = $drgroupname ;
	
	my $localevaname;
	($ret, $localevaname) = VCSAG_GET_ATTR_VALUE ("LocalEVAName", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute LocalEVAName is not configured properly", 3);
		return ERROR;
	}
	$href->{"LocalEVAName"} = $localevaname ;
	
	my $splittakeover;
	($ret, $splittakeover) = VCSAG_GET_ATTR_VALUE ("SplitTakeover", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute SplitTakeover is not configured properly", 4);
		return ERROR;
	}
	$href->{"SplitTakeover"} = $splittakeover ;

	my $autotakeover;
	($ret, $autotakeover) = VCSAG_GET_ATTR_VALUE ("AutoTakeover", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute AutoTakeover is not configured properly", 25);
		return ERROR;
	}
	$href->{"AutoTakeover"} = $autotakeover ;
	
	my $mgmtserver;
	($ret, $mgmtserver) = VCSAG_GET_ATTR_VALUE ("ManagementServer", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute ManagementServer is not configured properly", 5);
		return ERROR;
	}
	$href->{"ManagementServer"} = $mgmtserver ;
	
	my $username;
	($ret, $username) = VCSAG_GET_ATTR_VALUE ("UserName", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute UserName is not configured properly", 6);
		return ERROR;
	}
	$href->{"UserName"} = $username ;

	my $standbymgmtserver;
	($ret, $standbymgmtserver) = VCSAG_GET_ATTR_VALUE ("StandbyManagementServer", -1, 1, @ARGV);
	if($standbymgmtserver ne '') {
		$href->{"StandbyManagementServer"} = $standbymgmtserver ;
	}
	
	my $standbyusername;
	($ret, $standbyusername) = VCSAG_GET_ATTR_VALUE ("StandbyUserName", -1, 1, @ARGV);
	if($standbymgmtserver ne '') {
		$href->{"StandbyUserName"} = $standbyusername ;
	}

	return SUCCESS;
}

sub initialize()
{
	my ($self) = shift;
	
	$self->{'selectmanager_cmd'} = "\"select manager $self->{'mgmtserver'} username=$self->{'username'}\"";
	$self->{'selectlocalevasys_cmd'} = "\"select sys $self->{'localevaname'}\"";
	$self->{'lslocalevasys_cmd'} = "\"ls system $self->{'localevaname'}\"";
	$self->{'lsdrgroup_cmd'} = "\"ls dr_group $self->{'drgroupname'}\"";
	$self->{'failover_cmd'} = "\"set dr_group $self->{'drgroupname'} failover\"";

	my $retval = 0;
	my $bManagerChanged = 0;
	my @output_ref = ();
	my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'lslocalevasys_cmd'}";
	$retval= run_sssu_cmd($cmd, \@output_ref, 1, '\n');
	
	foreach my $error (@output_ref) {

		if ($error =~ /storage cell is managed by another agent/) {
			if(($self->{'standbymgmtserver'} ne '') && ($self->{'standbyusername'} ne '')) {
				VCSAG_LOG_MSG("N", "The [$cmd] failed with error [$error]. Will try to use the Standby Management server for sssu command execution.", 37, $cmd, $error); 
				$self->{'mgmtserver'} = $self->{'standbymgmtserver'};
				$self->{'username'} = $self->{'standbyusername'};
				$self->{'selectmanager_cmd'} = "\"select manager $self->{'mgmtserver'} username=$self->{'username'}\"";
				$bManagerChanged = 1;
			} else {
				VCSAG_LOG_MSG("I", "The $cmd failed with error $error. Please set the attributes for Stanby Management server.", 7, $cmd, $error); 
				return;
			}
	       }
		elsif ($error =~ /Failed to open password file/) {
			VCSAG_LOG_MSG("E", "Failed to open password file. Please make sure the password file exists and username username=$self->{'username'} is added to it.", 7, $self->{'username'}); 
			return;
	       }
		elsif ($error =~ /User credentials doesn't exist in password file/) {
			VCSAG_LOG_MSG("E", "Please check the attribute values and make sure the username username=$self->{'username'} is added to password file", 9, $self->{'username'}); 
			return;
		}
	}

	if($bManagerChanged) {
		my $retval = 0;
		my @output_ref = ();
	
		my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'}";
		$retval= run_sssu_cmd($cmd, \@output_ref, 1, '\n');

		foreach my $error (@output_ref) {

			if ($error =~ /Failed to open password file/) {
				VCSAG_LOG_MSG("E", "Failed to open password file. Please make sure the password file exists and username username=$self->{'username'} is added to it.", 7, $self->{'username'}); 
				return;
	       	}
			elsif ($error =~ /User credentials doesn't exist in password file/) {
				VCSAG_LOG_MSG("E", "Please check the attribute values and make sure the username username=$self->{'username'} is added to password file", 9, $self->{'username'}); 
				return;
			}
		}
	}

	$cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'selectlocalevasys_cmd'} $self->{'lsdrgroup_cmd'}" ;
	$retval = run_sssu_cmd($cmd, \@output_ref, 0, '\n');

	foreach my $q (@output_ref) {
		if ($q =~ /drmmode/) {
			my @role = split('\s+', $q);
			if($role[3] eq "source") {
				$self->{'role'} = ROLE_SOURCE;
			}
			elsif($role[3] eq "destination") {
				$self->{'role'} = ROLE_DESTINATION;
			}
			else {
				$self->{'role'} = ROLE_UNKNOWN;
			}
			next;
	       }
		if ($q =~ /scellname/) {
			my @remoteevaname = split('\s+', $q);
			$self->{'remoteevaname'} = $remoteevaname[3];
			next;
	       }
		if ($q =~ /writemode /) {
			my @mode = split('\s+', $q);
			if($mode[3] eq "synchronous") {
				$self->{'mode'} = MODE_SYNC;
			}
			elsif($mode[3] eq "asynchronous") {
				$self->{'mode'} = MODE_ASYNC;
			}
			else
			{
				$self->{'mode'} = MODE_UNKNOWN;
			}
			next;
	       }
	}

	$self->{'selectremoteevasys_cmd'} = "\"select sys $self->{'remoteevaname'}\"";
}

sub set_mode {
	my ($self) = shift;
	my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'selectlocalevasys_cmd'} $self->{'lsdrgroup_cmd'}";
	
	my @output = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');
       my @line = grep (/writemode /, @output );
	my @mode = split('\s+', $line[0]);

	#temp message, to be removed once agent is stable
	VCSAG_LOGDBG_MSG(1, "DR Mode $mode[3]", $mode[3]); 
	
	if($mode[3] eq "synchronous") {
		$self->{'mode'} = MODE_SYNC;
	}
	elsif($mode[3] eq "asynchronous") {
		$self->{'mode'} = MODE_ASYNC;
	}
	else
	{
		$self->{'mode'} = MODE_UNKNOWN;
	}
}

sub change_mode {
	my ($self) = shift;

       my $changeMode_cmd;
	if($self->{'mode'} == MODE_ASYNC)
	{	
		#change mode to synchronous
		$changeMode_cmd = "\"set dr_group $self->{'drgroupname'} writemode=synchronous\"";
	}
	elsif ($self->{'mode'} == MODE_SYNC)
	{	
		#change mode to asynchronous
		$changeMode_cmd = "\"set dr_group $self->{'drgroupname'} writemode=asynchronous\"";
		}
	
	my $selectsyscmd;
	if($self->{'role'} == ROLE_DESTINATION)	{
		$selectsyscmd = $self->{'selectremoteevasys_cmd'};
		}
	elsif($self->{'role'} == ROLE_SOURCE) {
		$selectsyscmd = $self->{'selectlocalevasys_cmd'};
	}

	my @tmparr = ();
	my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $selectsyscmd $changeMode_cmd";
	my $ret = run_sssu_cmd($cmd, \@tmparr, 0, '\n');

	foreach my $error (@tmparr) {
		if ($error =~ /Error: /) {
			VCSAG_LOG_MSG("E", "The command to change the write mode of DR Group $self->{'drgroupname'} failed with error: $error.", 8, "$self->{'drgroupname'}", $error); 
			return ERROR;
		}
	}
	return SUCCESS;
}

sub failover_required {
	my ($self) = shift;
	my $ret = FR_YES;
	
	#temp message, to be removed once agent is stable
	VCSAG_LOGDBG_MSG(1, "CHECKING IF FAILOVER REQUIRED");

	if($self->{'role'} == ROLE_DESTINATION)
	{
		#temp message, to be removed once agent is stable
		VCSAG_LOGDBG_MSG(1, "ROLE IS DESTINATION");
	
		#is link down and autotakeover set?
		if($self->linkdown())	{
			if($self->{'autotakeover'} == 0) {
				VCSAG_LOG_MSG("N", "The replication link for DR Group $self->{'drgroupname'} is down and AutoTakeover is set to 0; no action will be taken.", 10, "$self->{'drgroupname'}");
				$ret = FR_NO;
			}
			else {
				VCSAG_LOG_MSG("N", "The replication link for DR Group $self->{'drgroupname'} is down and AutoTakeover is set to 1; will try to takeover.", 11, "$self->{'drgroupname'}");
				$ret = FR_YES;
			}
		}
		
		#is replication suspended and forcetakover set?
		if($self->replication_suspended())	{
			if($self->{'splittakeover'} == 0) {
				VCSAG_LOG_MSG("N", "The replication for DR Group $self->{'drgroupname'} is suspended and SplitTakeover is set to 0; no action will be taken.", 12, "$self->{'drgroupname'}");
				$ret = FR_NO;
			}
			else {
				VCSAG_LOG_MSG("N", "The replication for DR Group $self->{'drgroupname'} is suspended and SplitTakeover is set to 1; will try to takeover.", 13, "$self->{'drgroupname'}");
				$ret = FR_YES;
			}
		}
	}
	return $ret;
}

sub wait_for_sync {
	my ($self) = shift;
	my ($oto) = split('\n', `$vcs_home/bin/hatype -value EVACA OnlineTimeout`);
	my $toval = $oto - 30;

	#check for full copy(eg initial sync, or when log is full) is going on and wait for it to complete
	if($self->fullcopy())
	{
		#wait for merge to complete
		my $cur_time = time();
		while (($cur_time - $self->{'start_time'} + 10) < $oto) {
			sleep(5);

			if($self->drmgroupcopystatecomplete() == 100)
			{
				VCSAG_LOG_MSG("I", "Full copy operation for $self->{'drgroupname'} is complete", 14, "$self->{'drgroupname'}"); 
				return SUCCESS;
			}
		}
		return ERROR;
	}	
	
	#if resync(eg sync after resuming replication, or when replication mode changed to sync)is going on
		
	my $logstate = $self->logstate();
	if($logstate eq "merging")
	{
		#wait for merge to complete
		my $cur_time = time();
		while (($cur_time - $self->{'start_time'} + 10) < $oto) {
			sleep(5);

			my $logstate = $self->logstate();
			if(($logstate eq "normal") && ($self->lefttransmit() == 0) )
			{
				VCSAG_LOG_MSG("I", "Log Merge operation for $self->{'drgroupname'} is complete", 15, "$self->{'drgroupname'}"); 
				return SUCCESS;
			}
		}
		return ERROR;
	}	
}

sub failover {
	my ($self) = shift;
	my $ret = F_FAILURE;
	if($self->linkdown() || $self->replication_suspended()){
		# data is consistent?(if full copy was going on before disaster)
		# From HP: In Synchronous mode, at destination if full copy is going on and copystatecomplete is not 100% and then if source
		# goes down the remaining date will be lost.But what ever data is transferred will be consistent at block level.I believe
		# layered applications will check for the consistency at the application level before application failover.Again if the original
		# source comes back then it is resynchronized with the new source and remaining data at the original source will be obsolete.

		my @tmparr = ();
		my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'selectlocalevasys_cmd'} $self->{'failover_cmd'}";
		my $ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');
		
		foreach my $error (@tmparr) {
			if ($error =~ /Error:/) {
				VCSAG_LOG_MSG("E", "The failover command for DR Group $self->{'drgroupname'} failed with error: $error.", 29, "$self->{'drgroupname'}", $error);
				return F_FAILURE;
			}
		}

		$self->scan_devices();
		return F_SUCCESS;
	}
	else {
		#check if secondary is uptodate. If not, wait for it to be uptodate
		
		#check if replication is async, if so change mode
		if($self->{'mode'} == MODE_ASYNC)
		{	
			$self->change_mode();
		}
		
		my $ret = $self->uptodate();
		
		if($ret == -1){
			VCSAG_LOG_MSG("E", "Cannot determine whether the replication is uptodate or not for DR Group $self->{'drgroupname'}. Administrative intervention required.", 27, "$self->{'drgroupname'}");
			return F_FAILURE;
		}elsif($ret == 0)	{
			VCSAG_LOG_MSG("N", "The secondary is not uptodate will wait for it to be uptodate", 18);
		
			if($self->wait_for_sync() == ERROR)	{
				VCSAG_LOG_MSG("W", "The OnlineTimeout is about to expire. The secondary is still not uptodate. Won't try failover the DR Group $self->{'drgroupname'}.", 16, "$self->{'drgroupname'}");
				return F_FAILURE;
			}
		}
				
		#else if (sync mode) failsafe conditions? => No need to check explicitely as wait_for_sync covers it generically
			
		#execute failover
		
		my @tmparr = ();
		my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'selectlocalevasys_cmd'} $self->{'failover_cmd'}";
		my $ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');
		
		foreach my $error (@tmparr) {
			if ($error =~ /Error:/) {
				VCSAG_LOG_MSG("E", "The failover command for DR Group $self->{'drgroupname'} failed with error: $error.", 29, "$self->{'drgroupname'}", $error);
				return F_FAILURE;
			}
		}

		$ret = F_SUCCESS;
		$self->scan_devices();
		
		#if we have changed the replication mode, revert it back
		if($self->{'mode'} == MODE_ASYNC)
		{
			$self->set_mode();
                     #we reached here because failover was successful. role is now source. To have change write mode to succeed we need to consider current role
			$self->{'role'} = ROLE_SOURCE;
			if($self->change_mode() == ERROR) {
				VCSAG_LOG_MSG("N", "Changing replication mode back to Asynchronous failed. Please use changeWriteMode action EP for the same.", 17);
			}
		}
		return $ret;
	}
}

#find out if the replication is suspended
sub replication_suspended {
	my ($self) = shift;
       
	my $suspendstate = $self->get_prop_value("suspendstate");
	
	if($suspendstate eq "suspend") {
		return 1;
	}
	return 0;
}

sub linkdown {
	my ($self) = shift;
	my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'selectlocalevasys_cmd'} $self->{'lsdrgroup_cmd'}";
	
	my @output = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');
        my @line = grep (/linkstate/, @output );
	my @linkstate = split('\s+', $line[0]);

	VCSAG_LOGDBG_MSG(1, "Link $linkstate[3]", $linkstate[3]);  

	
	if($linkstate[3] eq "failed") {
		return 1;
	}
	return 0;

}

sub logstate {
	my ($self) = shift;
	my $logstate = $self->get_prop_value("logstate");
	return $logstate;
}

sub fullcopy {
	my ($self) = shift;
	my $fullcopystate = $self->get_prop_value("fullcopystate");

	if($fullcopystate eq "yes") {
		return 1;
	}
	return 0;
}

sub drmgroupcopystatecomplete {
	my ($self) = shift;
	my $copypercent = $self->get_prop_value("drmgroupcopystatecomplete");

	return $copypercent;
}

sub lefttransmit {
	my ($self) = shift;
	my $lefttransmit = $self->get_prop_value("lefttransmit");

	return $lefttransmit;
}

sub uptodate {
	my ($self) = shift;
	if(($self->logstate() eq "") || ($self->lefttransmit() == -1) && ($self->fullcopy() == 0)) {
		return -1;
	}

	if(($self->logstate() eq "normal") && ($self->lefttransmit() == 0) && ($self->fullcopy() == 0)) {
		#temp message, to be removed once agent is stable
		VCSAG_LOGDBG_MSG(1, "The scondary is UPTODATE"); 
		return 1;
	}
	return 0;
}

sub replicationstatus {
	my ($self) = shift;
	my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'selectlocalevasys_cmd'} $self->{'lsdrgroup_cmd'}";
	
	my @output = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');
        my @line = grep (/operationalstate/, @output );
	my @operationalstate = split('\s+', $line[0]);
	return $operationalstate[3];
}

sub failsafe {
	my ($self) = shift;

	my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $self->{'selectlocalevasys_cmd'} $self->{'lsdrgroup_cmd'}" ;
	
	my @output = ();
	my @failsafe = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');

       my @line = grep (/failsafe/, @output );
	@failsafe = split('\s+', $line[0]);
		
	return $failsafe[3];
}

sub run_sssu_cmd {
	my ($cmd, $output_ref, $ignore_failure, $splitter) = @_;

	if($splitter eq '') {
		$splitter = '\s+';
	}

	my $home;
	my $userprofile;
	if($OSTYPE eq "WINDOWS"){
		$userprofile = $ENV{"USERPROFILE"};
		$ENV{"USERPROFILE"} = $WINUSRPROFILE;
	} else {
		$home = $ENV{'HOME'};
		if($OSTYPE eq "LINUX"){
			$ENV{"HOME"} = "/root";
		} else {
			$ENV{"HOME"} = "/";
		}
	}

	my $retval = 0;
	@{$output_ref} = split($splitter, `$cmd`);

	if ($? != 0) {
		$retval = $?/256;
		unless (1 == $ignore_failure ) {
			print "Command \'$cmd\' failed. The return value is $retval\n";
			VCSAG_LOG_MSG("W", "Command failed. The return value is $retval", 1007,
					"$retval");
		}
	} 

	if($OSTYPE eq "WINDOWS"){
		$ENV{"USERPROFILE"} = $userprofile;
	} else {
		$ENV{"HOME"} = $home;
	}

	return $retval;
}

#
#Gets the property values of DR group from Primary array
#
sub get_prop_value {
	my ($self) = shift;
	my ($property) = shift;
	
	my $selectsyscmd;
	if($self->{'role'} == ROLE_DESTINATION)	{
		$selectsyscmd = $self->{'selectremoteevasys_cmd'};
	}
	elsif($self->{'role'} == ROLE_SOURCE) {
		$selectsyscmd = $self->{'selectlocalevasys_cmd'};
	}

	my $cmd = "$self->{'sssupath'} $self->{'selectmanager_cmd'} $selectsyscmd $self->{'lsdrgroup_cmd'}" ;

	my @output = ();
	my @props = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');

	if($ret != 0) {
		return -1;
	}

       my @line = grep (/$property/, @output );
	@props = split('\s+', $line[0]);

	#temp message, to be removed once agent is stable
	VCSAG_LOGDBG_MSG(1, "The value of property $property is $props[3].", $property, $props[3]); 
	
	return $props[3];
}

sub scan_devices {
        my ($self) = shift;
        my $cmd;

	if ($OSTYPE eq "LINUX") {
                $cmd = "$VXDCTL enable";
		  VCSAG_LOG_MSG("N", "Issuing rescan command: $cmd", 28, "$cmd");
        }
        elsif ($OSTYPE eq "AIX")
        {
                $cmd="$CFGMGR";
		  VCSAG_LOG_MSG("N", "Issuing rescan command: $cmd", 28, "$cmd");
        }
        elsif ($OSTYPE eq "HPUX")
        {
                $cmd="$IOSCAN 2>&1 > $DEVNULL";
                VCSAG_LOG_MSG("N", "Issuing rescan command: $cmd", 28, "$cmd");
                my ($status, $output) = system($cmd);
                $cmd = "$VXDCTL enable";
        }
        elsif ($OSTYPE eq "SOLARIS")
        {
                $cmd="$DEVFSADM";
		  VCSAG_LOG_MSG("N", "Issuing rescan command: $cmd", 28, "$cmd");
        }

        my ($status, $output) = system($cmd);
        return 0;
}

#####
#fix for open EP. Temporarily added util functions here. 
#Should be removed in future when util functions are available in ArrayRep.pm
#copied relavant functions from SRDFAgent.pm
#####
sub get_lockfilename {
	my ($self) = shift;
	my $typ;
	my $lock_file;
	my $lock_dir = ""; 

	if (!defined($self->{'resource_type'}) || 
		(length($self->{'resource_type'}) == 0)) {

		$typ = `$ag_i18n_inc::HARES -value $self->{'resname'} Type`;
		chomp $typ;
	} else {
		$typ = $self->{'resource_type'};
	}

	if ($^O =~ /MSWin32/i) {
		$lock_dir = $ag_i18n_inc::VCSLOG . "\\lock\\" . $typ . "\\";
	} else {
		$lock_dir = $ag_i18n_inc::VCSLOG . "/lock/" . $typ . "/";
	}

	$lock_file = $lock_dir . ".VCS_" . $typ . "_" . $self->{'resname'} . 
		"_Online_Lockfile.DO-NOT-REMOVE";

	return $lock_file;
}

sub create_lockfile {
	my ($self) = shift;
	my ($lock_file) = $self->get_lockfilename();

	my (undef, $lock_dir, undef) = fileparse($lock_file);

	if (! -d $lock_dir) {
		mkdir  $lock_dir or 
			VCSAG_LOG_MSG("E", "Could not create lock directory $lock_dir",
				1001, "$lock_dir");
	}

	open(lfh, ">$lock_file") or 
		VCSAG_LOG_MSG("E", "Could not create file: $lock_file", 1002,
			 "$lock_file");

	close(lfh);
}

sub remove_lockfile {
	my($self) = shift;
	my ($lock_file) = $self->get_lockfilename();

	if (-f "$lock_file") {
		unlink("$lock_file") or 
			VCSAG_LOG_MSG("E", "Could not delete file: $lock_file", 1003,
				 "$lock_file");
	}
}


sub exists_lockfile {
	my ($self) = shift;
	my $retval = 0;
	my ($lock_file) = $self->get_lockfilename();

	if (-f "$lock_file") {
		$retval = 1;
	}
	return $retval;
}

#
# Return 1 if a parent resource of a parent service group is ONLINE
#

sub is_parent_online {
	my ($self) = shift;
	my ($p_res, $ret, $tmpstr, $sys);
	my @p_res_list = ();
	my $is_online = 0;

	$tmpstr = `$ag_i18n_inc::HASYS -nodeid`;
	chomp $tmpstr;
	$tmpstr =~ s/^.*Node name:[\s+](.*)[\s+]Node ID:.*$/$1/ ;
	$self->{'nodename'} = $sys = $tmpstr;

	# Are there any resources that depend on this resource
	@p_res_list = $self->get_res_deps();

	if ($#p_res_list >= 0) { # Parent resources were found
		foreach $p_res (@p_res_list) {

			$ret = system("$ag_i18n_inc::HARES -wait $p_res Probed 1 "
					." -sys $sys -time 120 2>$DEVNULL");
			if ($ret == 0) {
				my ($st) = split('\n', `$ag_i18n_inc::HARES -state $p_res -sys $sys`);		
				chomp $st;
				if ($st =~ /ONLINE/) {
					$is_online = 1;
					last;
				}
			}
		}
	} else { # No parent resources
		if ($self->is_parent_sg_res_online()) {
			$is_online = 1;	
		}
	}
	return $is_online;
}

#
# Return 1 if at least one OnOff/OnOnly resource from a parent service
# group is online.
#

sub is_parent_sg_res_online {
	my ($self) = shift;
	my ($sg_name, $p_sg_name, $line, $ret);
	my @lines = ();
	my @fields = ();
	my @valid_dep_lines = ();
	my $is_online = 0;
	my $sys = $self->{'nodename'};
	
	#
	# Get parent service groups that have an online local/online global
	# dependency to our service group
	#

	$sg_name = `$ag_i18n_inc::HARES -value $self->{'resname'} Group`;
	chomp $sg_name;

	(@lines) = split('\n', `$ag_i18n_inc::HAGRP -dep $sg_name`);
	chomp @lines;

	# VCS specific code --->
	(@valid_dep_lines) = grep (/online[\s+](local|global)/, @lines );

	foreach $line (@valid_dep_lines) {
		(@fields) = split('\s+', $line);

		if ($fields[1] eq $sg_name) {
			$p_sg_name = $fields[0];
		} else {
			next;
		}

		#
		# If at least one OnOff or OnOnly probed resource in the ParentSG
		# has gone online, then the SRDF resource can be online as well
		#

		$ret = system("$ag_i18n_inc::HAGRP -wait $p_sg_name Probed 1 -sys $sys"
				 . " -time 120 2>$DEVNULL");

		if ($ret == 0) {
			my @p_on_res_list = $self->get_res_list($p_sg_name, "ONLINE");	

			#
			# At least 1 online resource was found. Note: $#X == 0
			# implies @X has 1 element
			#
			if ($#p_on_res_list >= 0){

				#
				# We found at least one ONLINE resource. If it is of type 
				# OnOff or OnOnly, we can continue to remain ONLINE
				#   
				foreach my $p_res (@p_on_res_list) {
					my $p_type = `$ag_i18n_inc::HARES -value $p_res Type`;
					chomp $p_type;
					my $p_oprn_type = `$ag_i18n_inc::HATYPE -value $p_type Operations`;
					chomp $p_oprn_type;
					if ($p_oprn_type =~ /None/i ) {
						next;
					}

					$is_online = 1;
					last;
				}
			}
		}
		last if ($is_online == 1);
	}

	return $is_online;
}

#
# Returns the names of resources that belong to a given group and
# are in particular state
#
sub get_res_list {
	my ($self) = shift;
	my ($sg_name, $res_state) = @_;
	my @retarr = ();
	my @tmparr = ();
	my ($ret, $nodename, $resname);

	$ret = run_cmd("$ag_i18n_inc::HARES -list Group=$sg_name State=$res_state", 
				\@tmparr, 1);

	if ($ret) {
		foreach (@tmparr) {
			/^#/ && next;
			($resname, $nodename)  = split;

			if ((!grep(/^${resname}$/, @retarr)) &&
				($nodename =~ /$self->{'nodename'}/)) {

				push (@retarr, $resname);
			}
		}	
	}
	return @retarr;	
}


#
# Returns the names of parent resources
#
sub get_res_deps {
	my ($self) = shift;

	my ($sg_name, $res_state) = @_;
	my @retarr = ();
	my @tmparr = ();
	my ($ret, $grpname, $p_resname, $c_resname);

	$ret = run_cmd("$ag_i18n_inc::HARES -dep $self->{'resname'}", \@tmparr, 1);

	if ($ret) {
		foreach (@tmparr) {
			/^#/ && next;
			($grpname, $p_resname, $c_resname)  = split;
		
			#
			# Note: with VCSOne, the 'resname' would be <grpname>.<resname>
			# whereas the $c_resname would only have the trailing <resname>
			# part
			#
		
			if ((!grep(/^${p_resname}$/, @retarr)) && 
				( $c_resname =~ /$self->{'resname'}$/ )) {
				push (@retarr, $p_resname);
			}
		}	
	}
	
	return @retarr;	
}


sub run_cmd {
	my ($cmd, $output_ref, $ignore_failure) = @_;
	my $retval = 1;
	@{$output_ref} = `$cmd`;

	if ($? != 0) {
		$retval = 0;
		unless (1 == $ignore_failure ) {
			VCSAG_LOG_MSG("W", "Command: \'$cmd\' failed !", 1007,
					"$cmd");
		}
	} else {
		chomp @{$output_ref};
	}
	return $retval;
}

1;
