#
# $Copyright: Copyright (c) 2012 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $RCSfile: EVACASnap.pm,v $ $Revision: 1.1.2.7 $
#

package EVACASnap;

use ArrayRep;
@ISA = ("ArrayRep");

use EVACAAgent;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default %vmattrib);
@EXPORT = qw(
);

use ag_i18n_inc;

my $vcs_home=$VCSHOME;

my $VXDG = "/usr/sbin/vxdg";
my $VXDCTL = "/usr/sbin/vxdctl";
my $VXDISK = "/usr/sbin/vxdisk";
my $DGTYPE = "DiskGroup";
my $HARES = $vcs_home . "/bin/hares";
my $HATYPE = $vcs_home . "/bin/hatype";
my $HAGRP =  $vcs_home . "/bin/hagrp";

my $RECREATEVGTIMER = 3;    # If recreatevg fails,  how many times do we retry
my $RECREATEVGINTVL = 5; # If recreatevg fails, interval to wait before retrying
my $LSATTR = "lsattr";
my $LSPV = "lspv";

my $OSTYPE = "";
my $WINUSRPROFILE = "";

# General return codes
use constant SUCCESS => 0;
use constant ERROR => 1;


my $DEVNULL = "/dev/null";
if ($^O =~ /aix/i ){
	$OSTYPE = "AIX";
} elsif ($^O =~ /hpux/i ){
	$OSTYPE = "HPUX";
}elsif ($^O =~ /linux/i ){
	$OSTYPE = "LINUX";
}elsif ($^O =~ /solaris/i ){
	$OSTYPE = "SOLARIS";
}elsif ($^O =~ /MSWin32/i ){
	$OSTYPE = "WINDOWS";
	$DEVNULL = "nul";	

	#determine the user profile to be used according to the OS version.
	my @version = split('\s+' , `ver`);
	my $domain = Win32::DomainName();
	if($domain ne "") {
		$domain = "." . $domain;
	}
	#if version no. starts with 6, its win2k8
	if($version[4] =~ /^6/) {
		$WINUSRPROFILE = $ENV{"SYSTEMDRIVE"} . "\\Users\\Administrator" . $domain;		
	} else {
		$WINUSRPROFILE = $ENV{"SYSTEMDRIVE"} . "\\Documents and Settings\\Administrator" . $domain;		
	}
}


%default = {
	resname => '',
	
	#
	# For replicated DGs, target is the resname of EVACA resource.
	# For non-replicated DGs, target is name of the DiskGroup resource.
	#
	target  => '',

	# Type of snapshot
	sotype  => '',

	# Target is a replicated DG?
	nonrep_dg => '',

	# List of dgs dependent on me.
	dgname  => '',

	# EVACA resource object
	evacares => '',

	#hostname
	hostname => '',

	#No. of DR members
	nummembers => 0,

	#Volume manager type
	vmtype => ''
};

%vmattrib = (
	LVMVG     => 'VolumeGroup',
	DiskGroup => 'DiskGroup'
);


sub new {
	my($proto, $resname, $href) = @_;
	my (@dgname);
	my $evacares_name;
	my @tmp = "";
	my $index = 0;

	# DR group members
	my @fdsource_members = ();

	# PIT copy names
	my @pitcnames = ();

	my $class = ref($proto) || $proto;
	my $self = { %default };
	$self->{'start_time'} = time();
	$self->{'resname'} = $resname;
	$self->{'target'} = $href->{"TargetResName"};

	$self->{'sotype'} = "gold-only";

	$self->{'nonrep_dg'} = 0;

       # Get all the EVACA specific values and VxVM DG name from group variables
       my ($typ) = split ('\n', `$HARES -value $self->{'target'} Type`);
	my $dgattrib = "";	
	if ($typ eq "EVACA") {
		# Build the list of dependent VxVM DGs
		my (@depfields) = split('\n', `$HARES -dep $self->{'target'}`);
              foreach my $depline (@depfields) {
              	my (@deppair) = split('\s+', $depline);

			if ( $deppair[2] eq $self->{'target'} ) {
				@tmp = split( '\n', `$VCSHOME/bin/hares -value $deppair[1] Type` );
				if ( 0 == length $self->{'vmtype'} ) {
					$self->{'vmtype'} = $tmp[0];
					$dgattrib = $vmattrib{ $self->{'vmtype'} };
				} else {
					if ( $self->{'vmtype'} ne $tmp[0] ) {
						VCSAG_LOG_MSG( "E",	"Mixed VM types are not supported; exiting", 31 );
						die("Mixed VM types used in configuration");
					}
				}
				@tmp = split( '\n', `$VCSHOME/bin/hares -value $deppair[1] $dgattrib` );
				$dgname[$index] = $tmp[0];
				$index = $index + 1;
			}
              }

		$self->{'dgname'} = join(' ', "@dgname");

		# Set EVACA resource name
		$evacares_name = $self->{'target'};

	} else {
		# Case of non-replicated DiskGroup.
		$self->{'nonrep_dg'} = 1;

		# Get the target DiskGroup. 
		$self->{'vmtype'} = $typ;
              @tmp = split('\n', `$VCSHOME/bin/hares -value $self->{'target'} $vmattrib{$self->{'vmtype'}}`);
              $self->{'dgname'} = $tmp[0];

		# Set EVACA resource name (set to any evaca res in this VCS group).
		my ($sg) = split('\n', `$HARES -value $self->{'target'} Group`);
              my (@evacaresources) = split('\n', `$HARES -list Type=EVACA Group=$sg -localclus`);
              (@tmp) = split('\s+', $evacaresources[0]);

		$evacares_name = $tmp[0];
	}

	if (($self->{'vmtype'} eq "LVMVG") && ($OSTYPE !~ /AIX/i)) {
                VCSAG_LOG_MSG("E", "LVM is supported only on AIX; Exiting", 32);
                return undef;
       }

	my %evaca_attrs = ();

	@tmp = split('\n', `$HARES -value $evacares_name ManagementServer`);
	if( $? != 0) {
                VCSAG_LOG_MSG("E", "Error while retrieving the attribute values of EVACA resource specified in TargetResName. Please make sure that the EVACA resource name exists and is valid. Exiting.", 59);
		return undef;
	}
	$evaca_attrs{"ManagementServer"} = $tmp[0];

	@tmp = split('\n', `$HARES -value $evacares_name UserName`);
	if($? != 0) {
                VCSAG_LOG_MSG("E", "Error while retrieving the attribute values of EVACA resource specified in TargetResName. Please make sure that the EVACA resource name exists and is valid. Exiting.", 59);
                return undef;
       }
	$evaca_attrs{"UserName"} = $tmp[0];

	@tmp = split('\n', `$HARES -value $evacares_name LocalEVAName`);
	if($? != 0) {
                VCSAG_LOG_MSG("E", "Error while retrieving the attribute values of EVACA resource specified in TargetResName. Please make sure that the EVACA resource name exists and is valid. Exiting.", 59);
                return undef;
       }
	$evaca_attrs{"LocalEVAName"} = $tmp[0];

	@tmp = split('\n', `$HARES -value $evacares_name DRGroupName`);
	if($? != 0) {
                VCSAG_LOG_MSG("E", "Error while retrieving the attribute values of EVACA resource specified in TargetResName. Please make sure that the EVACA resource name exists and is valid. Exiting.", 59);
                return undef;
       }
	$evaca_attrs{"DRGroupName"} = $tmp[0];

	@tmp = split('\n', `$HARES -value $evacares_name SSSUPath`);
	if($? != 0) {
                VCSAG_LOG_MSG("E", "Error while retrieving the attribute values of EVACA resource specified in TargetResName. Please make sure that the EVACA resource name exists and is valid. Exiting.", 59);
                return undef;
       }
	$evaca_attrs{"SSSUPath"} = $tmp[0];

	$self->{'evacares'} = EVACAAgent->new($evacares_name, \%evaca_attrs);
	$self->{'evacares'}->initialize();

	#get hostname
	my @op = split('\s+', `$HASYS -nodeid`);
	$self->{'hostname'} = $op[5];

	bless($self, $class);
	return $self;
}

sub validate_args {
	my $href = shift;
	my $ret = SUCCESS; 
	
	my $target;
	($ret, $target) = VCSAG_GET_ATTR_VALUE ("TargetResName", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS)	{
		VCSAG_LOG_MSG("E", "The attribute TargetResName is not configured properly", 33);
		return ERROR;
	}
	$href->{"TargetResName"} = $target ;
	
	return SUCCESS;
}


sub DESTROY {}

sub run_sssu_cmd {
	my ($cmd, $output_ref, $ignore_failure, $splitter) = @_;

	if($splitter eq '') {
		$splitter = '\s+';
	}

	my $home;
	my $userprofile;
	if($OSTYPE eq "WINDOWS"){
		$userprofile = $ENV{"USERPROFILE"};
		$ENV{"USERPROFILE"} = $WINUSRPROFILE;
	} else {
		$home = $ENV{'HOME'};
		if($OSTYPE eq "LINUX"){
			$ENV{"HOME"} = "/root";
		} else {
			$ENV{"HOME"} = "/";
		}
	}

	my $retval = 0;
	@{$output_ref} = split($splitter, `$cmd`);

	if ($? != 0) {
		$retval = $?/256;
		unless (1 == $ignore_failure ) {
			VCSAG_LOG_MSG("W", "Command failed. The return value is $retval", 1007, "$retval");
		}
	} 

	if($OSTYPE eq "WINDOWS"){
		$ENV{"USERPROFILE"} = $userprofile;
	} else {
		$ENV{"HOME"} = $home;
	}

	return $retval;
}

sub _run_cmd {
	my ($self, $cmd) = @_;
	my $ret = SUCCESS;
	my (@out) = ();

	VCSAG_LOGDBG_MSG(1, "Executing cmd $cmd", "$cmd");

	@out = `$cmd`;
	$ret = $?;

	VCSAG_LOGDBG_MSG(1, "End _run_cmd return code $ret", "$ret");

	return $ret, @out;
}

sub take_snapshot {
	my $self = shift;
	my $ret = SUCCESS;
	my (@out) = "";
	my $evacares = $self->{'evacares'};
	my @pitcs = ();

	VCSAG_LOG_MSG("N", "Taking snapshots of the members of the DR Group $self->{'target'}.", 37, "$self->{'target'}");

	if($self->get_FDSource_members == ERROR) {
		return ERROR;
	}
	
	#Add snapshots for each of the DR group members
	for (my $i = 0; $i < $self->{'nummembers'}; $i++) {
		my $diskname = $self->{'fdsource_members'}[$i];

		VCSAG_LOGDBG_MSG(1, "VIRTUAL DISK $diskname", $diskname); 

	       #form the command
		my @tmparr = ();
		my $pitname = "SNAP-" . $diskname;
		my $snapcmd = "\"add snapshot $pitname vdisk=$diskname\"";
		my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $snapcmd";

		VCSAG_LOGDBG_MSG(1, "Add Snapshot command $cmd", $cmd); 

		#execute the command
		$ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');

		foreach my $error (@tmparr) {
			if ($error =~ /Error:/) {
				VCSAG_LOG_MSG("E", "The snapshot command for DR Group $evacares->{'drgroupname'} failed with error: $error.", 39, "$evacares->{'drgroupname'}", $error);
				return ERROR;
			}
		}
		$self->{'pitcnames'}[$i] = $pitname;
	}
	
	#present the pits to host
	$ret = $self->present_pitcs_to_host();

	if($ret == ERROR) {
		VCSAG_LOG_MSG("E", "Error while presenting the snapshot disks to the host for the DR group $evacares->{'drgroupname'}.", 40, "$evacares->{'drgroupname'}");
		return ERROR;
	}

	
	sleep(5);
	$evacares->scan_devices();

	VCSAG_LOG_MSG("I", "Snapshot for $self->{'target'} taken successfully", 41, "$self->{'target'}");

	return $ret;
}


sub suspend_replication {
	my ($self) = shift;
	my $ret = SUCCESS;
	my $evacares = $self->{'evacares'};

	#Replication can be suspended only from Primary.
	VCSAG_LOG_MSG("N", "Suspending the replication for DR group $evacares->{'drgroupname'}.", 42, "$evacares->{'drgroupname'}");

       #form the command
	my @tmparr = ();
	my $suspendcmd = "\"set dr_group $evacares->{'drgroupname'} suspend\"";
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectremoteevasys_cmd'} $suspendcmd";

	VCSAG_LOGDBG_MSG(1, "Suspend Replication command $cmd", $cmd); 

	#execute the command
	my $ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');

	foreach my $error (@tmparr) {
		if ($error =~ /Error:/) {
			VCSAG_LOG_MSG("E", "The suspend command for DR Group $self->{'drgroupname'} failed with error $error.", 43, "$self->{'drgroupname'}", $error);
			return ERROR;
		}
	}

	return SUCCESS;
}

sub resume_replication {
	my ($self) = shift;
	my $ret = SUCCESS;
	my $evacares = $self->{'evacares'};

	#Replication can be resumed only from Primary.
	VCSAG_LOG_MSG("N", "Resuming the replication for DR group $evacares->{'drgroupname'}.", 44, "$evacares->{'drgroupname'}");

       #form the command
	my @tmparr = ();
	my $suspendcmd = "\"set dr_group $evacares->{'drgroupname'} nosuspend\"";
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectremoteevasys_cmd'} $suspendcmd";

	#execute the command
	my $ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');

	foreach my $error (@tmparr) {
		if ($error =~ /Error:/) {
			VCSAG_LOG_MSG("E", "The resume command for DR Group $evacares->{'drgroupname'} failed with error $error.", 45, "$evacares->{'drgroupname'}", $error);
			return ERROR;
		}
	}

	return SUCCESS;
}

sub get_FDSource_members {
	my ($self) = shift;
	my $evacares = $self->{'evacares'};

	if ($self->{'nonrep_dg'} == 1) {
		#Get the member vdisks of the Diskgroup
		if($self->get_DiskGroup_members() == ERROR) {
			VCSAG_LOG_MSG("E", "Error while getting the vdisk members for the diskgroup $self->{'dgname'}.", 59, "$self->{'dgname'}");
			return ERROR;
		}
	} else {
		#Get the member vdisks of the DR group
		if($self->get_DRGroup_members() == ERROR) {
			VCSAG_LOG_MSG("E", "Error while getting the vdisk members of the DR group $evacares->{'drgroupname'}.", 38, "$evacares->{'drgroupname'}");
			return ERROR;
		}
	}
	return SUCCESS;
}

sub get_DRGroup_members {
	my ($self) = shift;
	my $evacares = $self->{'evacares'};
	
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $evacares->{'lsdrgroup_cmd'}" ;

	my @output = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');

	foreach my $error (@output) {
		if ($error =~ /Error:/) {
			VCSAG_LOG_MSG("E", "The command for getting the DR group members for DR Group $evacares->{'drgroupname'} failed with error $error.", 46, "$evacares->{'drgroupname'}", $error);
			return ERROR;
		}
	}

	my $i = 0;
	my $foundvdisk = 0;
	my @vdisks = ();
	my $diskname;
	foreach my $line (@output) {
		if ($line =~ /membervirtualdisks/) {
			$foundvdisk = 1;
			next;
              }
		if($line =~ /objectname/ && $foundvdisk) {
			my $pos = index($line , '\\');
			$diskname = substr($line , $pos);
			VCSAG_LOGDBG_MSG(1, "Virtual Disk Full Name $diskname", $diskname); 
			my @props = split(/\\/, $diskname);
			$vdisks[$i] = $props[$#props - 1];
			VCSAG_LOGDBG_MSG(1, "Virtual Disk Name $vdisks[$i]", $vdisks[$i]); 
			my $pitcname = "SNAP-" . $vdisks[$i];
			$self->{'pitcnames'}[$i] = $pitcname;

			$i = $i + 1;
		}
	}
	$self->{'nummembers'} = $i;
	VCSAG_LOGDBG_MSG(1, "Size of DR group members $self->{'nummembers'}", "$self->{'nummembers'}"); 

	$self->{'fdsource_members'} = \@vdisks;

	return SUCCESS;
}

sub get_DiskGroup_members {
	my ($self) = shift;
	my $evacares = $self->{'evacares'};

      	my (%vxdisk);
	my @DGdisks = ();
	my @vdisks = ();

	# Identify the disk(s) with the source disk group.
	# Once a snapshot is taken, the snapshot disk(s) will be the target disks,
	# which should be used when importing the disk group.

	# Get the vol mgr device names for the disk group

	my (@lines) = split('\n', `$VXDISK -o alldgs list \| grep $self->{'dgname'}`);
	my $i = 0;
	foreach my $l (@lines) {
		if($l !~ /clone_disk/) {
	      		my (@fields) = split('\s+', $l);
      			$DGdisks[$i++] = $fields[0];
		}
	}

	# Get uid of vdisk
	my %vmdisk = ();
	my $uid = "";
	my $disk = "";
	my @disk_info;
	my @lun_info;

	foreach $disk (@DGdisks) {
		my ($rc, @out) = $self->_run_cmd("$VXDISK -p list $disk");
		foreach my $line (@out) {
			if ($line =~ /LUN_SERIAL_NO/) {
				@lun_info = split(':', $line);
				$uid = ltrim($lun_info[1]);
				chomp($uid);
				$vmdisk{$disk} = $uid;
				my $test = $vmdisk{$disk};
				print "disk UID $test\n";
				VCSAG_LOGDBG_MSG(1, "disk UID $vmdisk{$disk} is for disk $disk", $vmdisk{$disk}, $disk); 
				$disk = "";
				$uid = "";
			}
		}
	}

	#get uid for each eva disk visible to this host
	#my %vdisks_to_lunids = ();
	my %vdisks_to_uids = ();

	my $vdiskname;
	my $disk_uid;

	my $evacares = $self->{'evacares'};

	my $lshostcmd = "\"ls host $self->{'hostname'}\"";
	
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $lshostcmd" ;

	VCSAG_LOGDBG_MSG(1, "Ls Host command $cmd", $cmd); 

	my @output = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');

	if($ret != 0) {
		return ERROR;
	}

	foreach my $line (@output) {
		if($line =~ /virtualdiskname/) {
			my $pos = rindex($line , '\\');
			$vdiskname = substr($line , $pos + 1);
			$disk_uid = $self->get_VDisk_property($vdiskname, "objecthexuid");
			$disk_uid =~ tr/-//d;
			$disk_uid = uc $disk_uid;
			$vdisks_to_uids{$disk_uid} = $vdiskname;
			VCSAG_LOGDBG_MSG(1, "The vdisk $vdisks_to_uids{$disk_uid} has uid $disk_uid", $vdisks_to_uids{$disk_uid}, $disk_uid); 
		}
	}

	#match the lun ids with dg disks lun ids to get the required vdisk names
	my @vdisks = ();
	$i = 0;
	foreach my $disk_name ( keys %vmdisk ) {
		my $disk_uid = $vmdisk{$disk_name};
		$vdisks[$i] = $vdisks_to_uids{$disk_uid};
		VCSAG_LOGDBG_MSG(1, "Virtual Disk Name $vdisks[$i]", $vdisks[$i]); 
		my $pitcname = "SNAP-" . $vdisks[$i];
		$self->{'pitcnames'}[$i] = $pitcname;
		$i++;
	}
	$self->{'nummembers'} = $i;
	$self->{'fdsource_members'} = \@vdisks;
}

sub get_VDisk_property {
	my ($self) = shift;
	my ($vdisk) = shift;
	my $propname = shift;
	my $evacares = $self->{'evacares'};
	my $lsvdisk_cmd = "\"ls vdisk \\" . qq("$vdisk\\) . qq("");
	VCSAG_LOGDBG_MSG(1, "Ls Vdisk sub command is $lsvdisk_cmd", $lsvdisk_cmd); 
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $lsvdisk_cmd" ;

	VCSAG_LOGDBG_MSG(1, "Ls Vdisk command for $vdisk is $cmd", $vdisk, $cmd); 

	my @output = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');

	foreach my $error (@output) {
		if ($error =~ /Error:/) {
			VCSAG_LOG_MSG("E", "The command for getting VDisk property for Vdisk $vdisk failed with error: $error.", 47, "$vdisk", $error);
			return ERROR;
		}
	}

       my @line = grep (/$propname/, @output );
	my @value = split('\s+', $line[0]);

	return $value[3];
}

sub get_snapshot_property {
	my ($self) = shift;
	my ($snapname) = shift;
	my $propname = shift;
	my $evacares = $self->{'evacares'};
	
	my $lssnap_cmd = "\"ls snapshot \\\"$snapname\\\"\"";
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $lssnap_cmd" ;

	VCSAG_LOGDBG_MSG(1, "Ls Snapshot command $cmd", $cmd); 

	my @output = ();
	my $ret = run_sssu_cmd($cmd, \@output, 0, '\n');

	foreach my $error (@output) {
		if ($error =~ /Error:/) {
			VCSAG_LOG_MSG("E", "The command for getting snapshot property for snapshot Vdisk $snapname failed with error: $error.", 48, "$snapname", $error);
			return ERROR;
		}
	}

       my @line = grep (/$propname/, @output );
	my @value = split('\s+', $line[0]);

	return $value[3];
}

sub present_pitcs_to_host {
	my ($self) = shift;
	my $ret = SUCCESS;
	my $evacares = $self->{'evacares'};

	for (my $i = 0; $i < $self->{'nummembers'}; $i++) {
		my $pitcname = $self->{'pitcnames'}[$i];

		VCSAG_LOG_MSG("N", "Presenting point-in-time $pitcname copy to host $self->{'hostname'}.", 49, "$pitcname", "$self->{'hostname'}");

       	#form the command
		my @tmparr = ();
		my $presentcmd = "\"add lun 0 host=$self->{'hostname'} vdisk=$pitcname\"";
		my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $presentcmd";

		VCSAG_LOGDBG_MSG(1, "Present Snapshot command $cmd", $cmd); 

		#execute the command
		my $ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');

		foreach my $error (@tmparr) {
			if ($error =~ /Error:/) {
				VCSAG_LOG_MSG("E", "The present command for point-in-time copy $pitcname failed with error $error.", 50, $pitcname, $error);
				return ERROR;
			}
		}
	}

	return SUCCESS;
}

sub delete_snapshots {
	my ($self) = shift;
	my $pitname = shift;
	my $ret = SUCCESS;
	my $evacares = $self->{'evacares'};

	#delete snapshots for each of the DR group members
	for (my $i = 0; $i < $self->{'nummembers'}; $i++) {
		my $pitcname = $self->{'pitcnames'}[$i];
		
		VCSAG_LOG_MSG("N", "Deleting snapshot $pitcname .", 51, "$pitcname");

       	#form the command
		my @tmparr = ();
		my $deletecmd = "\"delete vdisk $pitcname\"";
		my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $deletecmd";

		VCSAG_LOGDBG_MSG(1, "Delete Snapshot command $cmd", $cmd); 

		#execute the command
		my $ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');

		foreach my $error (@tmparr) {
			if ($error =~ /Error:/) {
				VCSAG_LOG_MSG("E", "The delete snapshot command for snapshot disk $pitcname failed with error $error.", 52, "$pitcname", $error);
				return ERROR;
			}
		}
	}

	return SUCCESS;
}

sub unpresent_pitcs_to_host {
	my ($self) = shift;
	my $ret = SUCCESS;
	my $evacares = $self->{'evacares'};

	#get lun id for each snapshot
	my %snapshots_to_lunids = ();
	my $snapname;
	my $lunnumber;
	my $fullhostname;

	my $evacares = $self->{'evacares'};

	my $lshostcmd = "\"ls host $self->{'hostname'}\"";
	
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $lshostcmd" ;

	VCSAG_LOGDBG_MSG(1, "Ls Host command $cmd", $cmd); 

	my @output = ();
	$ret = run_sssu_cmd($cmd, \@output, 0, '\n');

	if($ret != 0) {
		return ERROR;
	}

	foreach my $line (@output) {
		if ($line =~ /objectname/) {
			my @temp = split('\s+', $line);
			$fullhostname = $temp[3];
              }
		if ($line =~ /lunnumber/) {
			my @temp = split('\s+', $line);
			$lunnumber = $temp[3];
              }
		if($line =~ /virtualdiskname/) {
			my $pos = rindex($line , '\\');
			$snapname = substr($line , $pos + 1);
			VCSAG_LOGDBG_MSG(1, "The vdisk $snapname has lun_id $lunnumber", $snapname, $lunnumber); 
			$snapshots_to_lunids{$snapname} = $lunnumber;
		}
	}

	my $lun_no;
	my $lunname;
	#delete snapshots for each of the DR group members
	for (my $i = 0; $i < $self->{'nummembers'}; $i++) {
		my $pitcname = $self->{'pitcnames'}[$i];
		$lun_no = $snapshots_to_lunids{$pitcname};
		$lunname = $fullhostname . "\\" .$lun_no;
		
		VCSAG_LOG_MSG("N", "Un-presenting the lun $lunname for point-in-time $pitcname copy from host $self->{'hostname'}.", 53, "$lunname", "$pitcname", "$self->{'hostname'}");

       	#form the command
		my @tmparr = ();
		my $unpresentcmd = "\"delete lun $lunname\"";
		my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $unpresentcmd";

		VCSAG_LOGDBG_MSG(1, "Unpresent Vdisk command $cmd", $cmd); 

		#execute the command
		my $ret = run_sssu_cmd($cmd, \@tmparr, 1, '\n');

		foreach my $error (@tmparr) {
			if ($error =~ /Error:/) {
				VCSAG_LOG_MSG("E", "The unpresent command for the lun $lunname for point-in-time copy $pitcname failed with error $error.", 54, $lunname, $pitcname, $error);
				return ERROR;
			}
		}
	}

	return SUCCESS;
}

sub pitc_gold {
	my ($self) = @_;
	my $ret = SUCCESS;

	VCSAG_LOG_MSG("N", "Initiating GOLD Firedrill on target $self->{'target'}.", 55, "$self->{'target'}");

	if ($self->{'nonrep_dg'} == 0) {
		$ret = $self->suspend_replication();
		if ($ret == ERROR) {
			return ERROR;
		}
	}

	$ret = $self->take_snapshot();
	if ($ret) {

		# Resume the replication that we just stopped.
		if ($self->{'nonrep_dg'} == 0) {
			$self->resume_replication();
		}

		VCSAG_LOG_MSG("E", "Failed to take a snapshot of the target $self->{'target'}; exiting.", 56, "$self->{'target'}");
		return ERROR;
	} 

	my (@fields) = split('\s+', `$vcs_home/bin/hasys -nodeid`);
	system("$HARES -modify $self->{resname} Responsibility $fields[5]");

	# Don't care about the return value.
	if ($self->{'nonrep_dg'} == 0) {
		$self->resume_replication();
	}

	if ( $self->{'vmtype'} eq "DiskGroup" ) {
		$ret = $self->mount_diskgroup("gold");
	} elsif ( $self->{'vmtype'} eq "LVMVG" ) {
		$ret = $self->mount_snapshot_lvm("gold");
	}

	return $ret;
}


sub pitc {
	my ($self) = @_;
	my $ret = SUCCESS;

	if ($self->{'nonrep_dg'} == 0) {
		if ($self->{'evacares'}->fullcopy()) {
			VCSAG_LOG_MSG("E", "Full copy is going on. Firedrill requires replication to be in consistent state. Exiting.",	57);
			return 1;
		}
	}

	$ret = $self->pitc_gold();

	return $ret;
}


#
# For vxvm as the underlying volume manager, the subroutine will clear
# import information on the disks in the device group, update the udid
# of the disks and set an appropriate tag on the disks so that when the
# vxdg import cmd is fired for the target diskgroup and snapshot diskgroup
# with the appropriate cli options, it will succeed.
# Returns: 0 on success
#	   1 on failure
#
sub EVACA_prepare_disks {
	my ($self) = shift;
	my $ret = SUCCESS;

	my ($tagname) = shift; # We should be able to return the 
				# tagname value to the caller this way
				# tagname is a pass-by-reference argument
	my ($cnt) = 0;
	my (@disks);
	my (@vdisk) = ();
	my (@vdisk_uid) = ();
	my ($vd);
	my (@out) = ();
	my $rc = 0;

	my $tag = "SNAP-";

	#
	# For gold, we need to prepare the snapshot disks. 
	# For silver and bronze we need to prepare the target disks.
	#
	
	# get the objecthexuid of snapshot
	for (my $i = 0; $i < $self->{'nummembers'}; $i++) {
		my $pitcname = $self->{'pitcnames'}[$i];
		my $uid = $self->get_snapshot_property($pitcname, "objecthexuid");
		$uid =~ tr/-//d;
		$uid = uc $uid;
		$vdisk_uid[$i] = $uid;
		VCSAG_LOGDBG_MSG(1, "UID for snapshot $pitcname is $uid", $pitcname, $uid);
	}

	
	# Get uid of vdisk
	my %vmdisk = ();
	my $uid = "";
	my $disk = "";
	my @disk_info;
	my @lun_info;

	system("$VXDISK scandisks");

	my @output = split('\n' , `$VXDISK -p list`);
	foreach my $line (@output) {
		#VCSAG_LOGDBG_MSG(1, "Vxdisk_-p_list line $line", $line);
		if ($line =~ /DISK\s+/) {
			@disk_info = split(':', $line);
			$disk = ltrim($disk_info[1]);
			chomp($disk);
			#VCSAG_LOGDBG_MSG(1, "Vxdisk_-p_list DISK $disk", $disk);
		} 
		elsif ($line =~ /^LUN_SERIAL_NO\s+/) {
			@lun_info = split(':', $line);
			$uid = ltrim($lun_info[1]);
			chomp($uid);
			$vmdisk{"$uid"} = "$disk";
			#VCSAG_LOGDBG_MSG(1, "Vxdisk_-p_list UID $uid is for disk $disk", $uid, $disk); 
		}
	}

	# Now get the disks with vdisk's uid matching in the disk array
	my $cnt = 0;
	foreach $vd (@vdisk_uid) {
		#
		# Get the VxVM disk which has last 32 characters 
		# of the disk udid matching uid
		#
		$disks[$cnt] = $vmdisk{"$vd"};
		VCSAG_LOGDBG_MSG(1, "The Snapshot Disk is $disks[$cnt]", $disks[$cnt]); 
		$cnt++;
	}

	# set the tagname passed by reference
	$tagname = $tag . $self->{'target'};

	my $final_tag = $tag . $self->{'target'};

	# In the future, there should be code to check the type of
	# volume manager technology before calling the appropriate
	# volume-manager-specific routine to set up the disks for an
	# import operation.
	$ret = ArrayRep::vxvm_prepare_disks($final_tag, 0, @disks);

	return $ret;
}

#
# Arguments to the subroutine:
# ----------------------------
# self: Resource instance
# case: Type of firedrill: gold/silver/bronze
#
# Return Values:
# --------------
# 0 on success
# 1 on failure
#
# Functionality:
# --------------
# The subroutine sets up everything for a successful import of the
# diskgroup on the right set of disks: FlashCopy-luns if gold firedrill and
# Target-luns in case of silver and bronze firedrills.
#
sub mount_diskgroup {
	my ($self, $case) = @_;
	my $ret = SUCCESS;
	my $evacares = $self->{'evacares'};

	#
	# Clear any import info, update udid, set tag on the appropriate disks.
	# Basically setup the disks for the impending dg import.
	#	
	my $tagname = "";

	# tagname will be passed by reference into the prepare_disks sub
	$ret = $self->EVACA_prepare_disks($case, \$tagname);

	if ($ret) {
		#
		# An appropriate error message would already have been logged
		# in the sub prepare_disks
		#
		return ERROR;
	}

	# Test import of the dg
	# In the future, there should be code to check the type of
	# volume manager technology before calling the appropriate
	# volume-manager-specific routine to test that the import
	# operation succeeds.
	$ret = ArrayRep::vxvm_test_diskgroup_import( $self->{'dgname'}, $case, $tagname);

	return $ret;
}

sub mount_snapshot_lvm {
	my ( $self, $case ) = @_;
	my $ret = SUCCESS;
	my (@SIdisks);
	my ( $vgname, $disk );
	my @cmds = ();

	# Map VG names to VG ids
	my %vgs_to_id = ();

	# Map VGid to all its Snapshot disks
	my %vgid_to_disks = ();

	my $RECREATEVG = "/usr/sbin/recreatevg";
	my $CHDEV      = "/usr/sbin/chdev";

	#get all harddisk names and lun ids using lvm command

	my %disks_to_lunids = ();
	my $diskname;
	my @disknames = ();
	my $lunid;
	my $decval;
	my $lun_no;
	my $line;
	my $i = 0;
	my @output = ();
	my ($rc, @out) = $self->_run_cmd($LSPV);
	
	foreach $line (@out) {
		@output = split('\s+', $line);	
		$disknames[$i++] = $output[0];
	}
		
	foreach $diskname (@disknames) {
		@out = `$LSATTR -l $diskname -E -a lun_id`;
		@output = split(' ', $out[0]);

		$lunid = $output[1];
		$decval = hex($lunid);
		$lun_no = $decval >> 48;
		VCSAG_LOGDBG_MSG(1, "The Disk $diskname has lun_id $lun_no", $diskname, $lun_no); 
		$disks_to_lunids{$lun_no} = $diskname;
	}
	
	#get lun id for each snapshot
	my %snapshots_to_lunids = ();
	my $snapname;
	my $lunnumber;

	my $evacares = $self->{'evacares'};

	my $lshostcmd = "\"ls host $self->{'hostname'}\"";
	
	my $cmd = "$evacares->{'sssupath'} $evacares->{'selectmanager_cmd'} $evacares->{'selectlocalevasys_cmd'} $lshostcmd" ;

	VCSAG_LOGDBG_MSG(1, "Ls Host command $cmd", $cmd); 

	@output = ();
	$ret = run_sssu_cmd($cmd, \@output, 0, '\n');

	if($ret != 0) {
		return ERROR;
	}

	foreach my $line (@output) {
		if ($line =~ /lunnumber/) {
			my @temp = split('\s+', $line);
			$lunnumber = $temp[3];
              }
		if($line =~ /virtualdiskname/) {
			my $pos = rindex($line , '\\');
			$snapname = substr($line , $pos + 1);
			VCSAG_LOGDBG_MSG(1, "The vdisk $snapname has lun_id $lunnumber", $snapname, $lunnumber); 
			$snapshots_to_lunids{$snapname} = $lunnumber;
		}
	}

	#match the lun ids with lvm disks lun ids to get the mapped disk names
	for (my $i = 0; $i < $self->{'nummembers'}; $i++) {
		my $pitcname = $self->{'pitcnames'}[$i];
		$lun_no = $snapshots_to_lunids{$pitcname};
		$SIdisks[$i] = $disks_to_lunids{$lun_no};
		VCSAG_LOGDBG_MSG(1, "The harddisk name for PITC $pitcname is $SIdisks[$i]", $pitcname, $SIdisks[$i]); 
	}

	my @vgnames = split( ' ', $self->{'dgname'} );

	# Construct a hash to map VGname to VG serial id
	ArrayRep::aix_lvm_get_vgs_id( \@vgnames, \%vgs_to_id );

	my @vgids = ( values %vgs_to_id );

	# Construct a hash to map VGids to SI disks
	ArrayRep::aix_lvm_get_vgdisks( \@vgids, \%vgid_to_disks, \@SIdisks );

	foreach $vgname ( keys %vgs_to_id ) {
		my $Lv_Prefix = "fd_";

		my $fdvgname = $vgname . "_fd";
		my @disks    = @{ $vgid_to_disks{ $vgs_to_id{$vgname} } };

		foreach $disk (@disks) {
			push( @cmds, "$CHDEV -l $disk -a pv=clear" );
			$ret = ArrayRep::ar_run_cmds( \@cmds );

			if ($ret) {
				VCSAG_LOG_MSG( "W", "Could not clear PV id on $disk. recreatevg might fail ", 134, "$disk" );
			}
		}

		# At times, recreatevg fails when called immediately after a split.
		# So, sleep for  $RECREATEVGINTVL seconds before trying recreatevg

		for ( my $i = $RECREATEVGTIMER ; $i > 0 ; $i-- ) {
			push( @cmds, "$RECREATEVG -y $fdvgname -Y $Lv_Prefix -n @disks" );
			$ret       = ArrayRep::ar_run_cmds( \@cmds );
			$Lv_Prefix = "NA";
			last if ( !$ret );
			VCSAG_LOG_MSG( "I",
				"recreatevg failed; retrying again after $RECREATEVGINTVL seconds; $i more retries left",
				137, "$RECREATEVGINTVL", "$i" );
			push( @cmds, "sleep $RECREATEVGINTVL " );
			ArrayRep::ar_run_cmds( \@cmds );
		}

		if ($ret) {
			VCSAG_LOG_MSG( "E", "Cannot recreate VG $fdvgname; exiting",
				139, "$fdvgname" );
			return ERROR;
		}

		my $line = `$HARES -list VolumeGroup=$fdvgname`;
		my ( $lvmresname, undef ) = split( /\s+/, $line );

		ArrayRep::aix_lvm_update_majornum_attrib( $lvmresname, \@disks );
	}

	return 0;
}



sub destroy_snapshots {
	my ($self) = @_;
	my $evacares = $self->{'evacares'};
	my $ret = SUCCESS;

	my @cmds   = ();

	if ( $self->{'vmtype'} eq "LVMVG" ) {
		my @dgs = map ( $_ . "_fd", ( split( /\s+/, $self->{'dgname'} ) ) );
		$ret = ArrayRep::aix_lvm_exportvgs( \@dgs );

		if($ret != 0) {
			VCSAG_LOG_MSG( "W", "The exportvg commands failed. Please analyse the failures and try the operation outside of VCS control again as required.", 58);
		}
	}
	
	#get source EVA disks for snapshots
	if($self->get_FDSource_members == ERROR) {
		return ERROR;
	}

	$ret = $self->unpresent_pitcs_to_host();
	
	if ($ret) {
		return ERROR;
	}

	$ret = $self->delete_snapshots();

	if ($ret) {
		return ERROR;
	} else {
		my (@fields) = split( '\s+', `$VCSHOME/bin/hasys -nodeid` );
		system("$VCSHOME/bin/hares -modify $self->{'resname'} Responsibility \"\"");
	}

	sleep(5);
	$evacares->scan_devices();

	return $ret;
}


# Left trim function to remove leading whitespace
sub ltrim($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	return $string;
}
